package com.nokia.finance.tasks.service.car;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.TypeFactory;
import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.pojo.po.common.AreaPo;
import com.nokia.finance.tasks.pojo.po.common.OrganizationPo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
@Service
public class CarService {
    /**
     * 匹配车牌省份
     */
    static final Pattern HAS_CHE_PAI_PROVINCE_PATTERN = Pattern.compile("[" + Pattern.quote("京津晋冀蒙辽吉黑沪苏浙皖闽赣鲁豫鄂湘粤桂琼渝川贵云藏陕甘青宁国防") + "]");
    /**
     * 匹配非车牌字符
     */
    static final Pattern NOT_CHE_PAI_PATTERN = Pattern.compile("[^京津晋冀蒙辽吉黑沪苏浙皖闽赣鲁豫鄂湘粤桂琼渝川贵云藏陕甘青宁新港澳学挂领试超练警国防A-Z\\d]");
    /**
     * 车牌正则
     */
    static final Pattern CHE_PAI_PATTERN = Pattern.compile(
            "([京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领A-Z][A-Z](([DF]((?![IO])[A-Z0-9](?![IO]))\\d{4})|(\\d{5}[DF]))|[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领A-Z][A-Z][A-Z0-9]{4}[A-Z0-9挂学警港澳])"
    );

    /**
     * 一级单位的二级单位字典
     */
    static final Map<String, List<String>> ER_JI_MAP;

    static {
        try {
            ER_JI_MAP = new ObjectMapper().readValue(
                    """
{
    "石家庄": ["鹿泉", "藁城", "栾城", "井陉矿区", "井陉", "无极", "正定", "元氏", "新乐", "晋州", "平山", "灵寿", "赞皇", "赵县", "行唐", "高邑", "辛集", "深泽"],
    "唐山": ["唐山高开区", "迁西", "海港", "开平", "丰南", "滦县", "乐亭", "丰润", "玉田", "古冶", "曹妃甸", "遵化", "滦南", "迁安"],
    "秦皇岛": ["北戴河新区", "北戴河", "山海关", "昌黎", "卢龙", "青龙", "抚宁"],
    "邯郸": ["曲周", "魏县", "馆陶", "磁县", "大名", "鸡泽", "成安", "涉县", "永年", "武安", "峰峰", "广平", "临漳", "邱县", "肥乡"],
    "邢台": ["新河", "南宫", "隆尧", "内邱", "平乡", "宁晋", "广宗", "清河", "临西", "任县", "巨鹿", "沙河", "威县", "临城", "柏乡", "南和"],
    "保定": ["涞水", "蠡县", "顺平", "博野", "安国", "涞源", "唐县", "定州", "高阳", "曲阳", "阜平", "清苑", "高碑店", "满城", "涿州", "易县", "望都", "徐水", "定兴", "白沟"],
    "张家口": ["张北", "崇礼", "康保", "赤城", "阳原", "万全", "下花园", "尚义", "怀安", "怀来", "蔚县", "涿鹿", "沽源", "宣化"],
    "承德": ["承德县", "兴隆", "宽城", "平泉", "营子", "隆化", "滦平", "围场", "丰宁", "双滦"],
    "廊坊": ["文安", "霸州", "大城", "廊坊开发区", "三河", "香河", "永清", "胜芳", "燕郊", "固安", "大厂"],
    "沧州": ["东光", "吴桥", "黄骅", "盐山", "孟村", "泊头", "献县", "南皮", "渤海新区", "海兴", "沧县", "河间", "青县", "任丘", "肃宁"],
    "衡水": ["景县", "阜城", "枣强", "深州", "饶阳", "故城", "武强", "武邑", "冀州", "安平"],
    "雄安": ["容城", "雄县", "安新"]
}
                            """,
                    TypeFactory.defaultInstance().constructMapType(Map.class, String.class, List.class)
            );
        } catch (JsonProcessingException e) {
            throw new MyRuntimeException(e);
        }
    }

    /**
     * 匹配车牌
     * @param chePai 车牌
     */
    public String getChePai(String chePai) {
        if (!StringUtils.hasText(chePai)) {
            return "";
        }
        // 字母转大写
        String upperCase = chePai.toUpperCase();
        // 删除非车牌字符
        String s = NOT_CHE_PAI_PATTERN.matcher(upperCase).replaceAll("");
        Matcher m = CHE_PAI_PATTERN.matcher(s);
        if (m.find()) {
            return m.group(0);
        }
        if (HAS_CHE_PAI_PROVINCE_PATTERN.matcher(chePai).find()) {
            log.warn("车牌匹配失败: {} -> {}", chePai, s);
            return s;
        }
        log.warn("车牌匹配失败: {} -> {}", chePai, upperCase);
        return upperCase;
    }

    /**
     * 车牌是否匹配失败
     * @param chePai 车牌
     */
    public String chePaiFail(String chePai) {
        if (!StringUtils.hasText(chePai)) {
            return "1";
        }
        // 字母转大写删除非车牌字符
        String s = NOT_CHE_PAI_PATTERN.matcher(chePai.toUpperCase()).replaceAll("");
        Matcher m = CHE_PAI_PATTERN.matcher(s);
        if (m.find()) {
            return "0";
        }
        return "1";
    }

    /**
     * 车牌是否包含报废
     * @param chePai 车牌
     */
    public String baoFei(String chePai) {
        if (chePai.contains("报废")) {
            return "1";
        }
        return "0";
    }

    /**
     * 获取一级单位
     * @param unit 单位
     */
    public String getFirstUnit(String unit) {
        if (!StringUtils.hasText(unit)) {
            return "";
        }
        if (unit.contains("机动通信局") || unit.contains("机动局")) {
            return "机动局";
        }
        if (unit.contains("雄安基地建设部")) {
            return "雄安基地建设部";
        }
        if (unit.contains("华北基地建设部")) {
            return "华北基地建设部";
        }
        for (String yj : ER_JI_MAP.keySet()) {
            if (unit.contains(yj)) {
                return yj;
            }
        }
        return "省公司本部";
    }

    /**
     * 获取而二级单位
     * @param unit 单位
     * @param firstUnit 一级单位
     */
    public String getSecondUnit(String unit, String firstUnit) {
        if (!StringUtils.hasText(unit)) {
            return "";
        }
        if ("省公司本部".equals(firstUnit)) {
            return firstUnit;
        }
        if ("机动局".equals(firstUnit)) {
            for (String yj : ER_JI_MAP.keySet()) {
                if (unit.contains(yj)) {
                    return "机动局" + yj;
                }
            }
            return "机动局本部";
        }
        if ("石家庄".equals(firstUnit)) {
            if (unit.contains("开发区")) {
                return "石家庄开发区";
            }
        }
        if ("廊坊".equals(firstUnit)) {
            if (unit.contains("开发区")) {
                return "廊坊开发区";
            }
        }
        if ("邢台".equals(firstUnit)) {
            if (unit.contains("内丘")) {
                return "内邱";
            }
            if (unit.contains("任泽")) {
                return "任县";
            }
        }
        if ("唐山".equals(firstUnit)) {
            if (unit.contains("高开区")) {
                return "唐山高开区";
            }
            if (unit.contains("滦州")) {
                return "滦县";
            }
        }
        List<String> ejs = ER_JI_MAP.get(firstUnit);
        if (CollectionUtils.isEmpty(ejs)) {
            return firstUnit;
        }
        if ("雄安".equals(firstUnit)) {
            unit = unit.replace("雄安新区", "");
        }
        for (String ej : ejs) {
            if (unit.contains(ej)) {
                return ej;
            }
        }
        return firstUnit + "本部";
    }

    /**
     * 获取三级单位
     * @param unit 单位
     * @param secondUnit 二级单位
     */
    public String getThirdUnit(String unit, String secondUnit) {
        if (!StringUtils.hasText(unit)) {
            return secondUnit;
        }
        String[] a = unit.split("_");
        if (a.length == 1) {
            return unit;
        }
        if (a.length < 4) {
            return secondUnit;
        }
        return a[3];
    }

    /**
     * 获取二级组织机构编码
     * @param secondOrgs 二级组织机构列表
     * @param unit 单位
     */
    public String getAreaNo(List<OrganizationPo> secondOrgs, String unit) {
        if (!StringUtils.hasText(unit)) {
            return "";
        }
        if (unit.contains("机动通信局") || unit.contains("机动局") || unit.contains("传输局")) {
            return "-11";
        }
        if (unit.contains("省公司本部") || unit.contains("雄安基地建设部") || unit.contains("华北基地建设部")) {
            return "-12";
        }
        for (OrganizationPo secondOrg : secondOrgs) {
            if (unit.contains(secondOrg.getName())) {
                return secondOrg.getId();
            }
        }
        return "-12";
    }

    /**
     * 获取组织机构名称
     * @param orgMap 组织机构字典
     * @param orgNo 组织机构编码
     */
    public String getOrgName(Map<String, OrganizationPo> orgMap, String orgNo) {
        if (!StringUtils.hasText(orgNo)) {
            return "";
        }
        OrganizationPo po = orgMap.get(orgNo);
        if (po != null) {
            return po.getName();
        }
        return "";
    }

    /**
     * 获取三级组织机构编码
     * @param thirdOrgListMap 三级组织机构列表字典
     * @param areaNo 二级组织机构编码
     * @param areaName 二级组织机构名称
     * @param unit 单位
     */
    public String getCityNo(Map<String, List<OrganizationPo>> thirdOrgListMap, String areaNo, String areaName,
                            String unit) {
        if (!StringUtils.hasText(areaNo) || !StringUtils.hasText(areaName)) {
            return "";
        }
        if ("石家庄".equals(areaName)) {
            if (unit.contains("井陉矿区")) {
                return "D0130185";
            }
            if (unit.contains("井陉")) {
                return "D0130121";
            }
        }
        if ("秦皇岛".equals(areaName)) {
            if (unit.contains("北戴河新区")) {
                return "D0130325";
            }
            if (unit.contains("北戴河")) {
                return "D0130304";
            }
        }
        if ("邯郸".equals(areaName)) {
            if (unit.contains("峰峰")) {
                return "D0130406";
            }
        }
        if ("邢台".equals(areaName)) {
            if (unit.contains("内丘")) {
                return "D0130523";
            }
            if (unit.contains("任泽")) {
                return "D0130526";
            }
        }
        if ("省机动局".equals(areaName)) {
            if (unit.contains("沧州")) {
                return "HECS180";
            }
            if (unit.contains("唐山")) {
                return "HECS181";
            }
            if (unit.contains("秦皇岛")) {
                return "HECS182";
            }
            if (unit.contains("廊坊")) {
                return "HECS183";
            }
            if (unit.contains("张家口")) {
                return "HECS184";
            }
            if (unit.contains("邢台")) {
                return "HECS185";
            }
            if (unit.contains("邯郸")) {
                return "HECS186";
            }
            if (unit.contains("保定")) {
                return "HECS187";
            }
            if (unit.contains("石家庄")) {
                return "HECS188";
            }
            if (unit.contains("承德")) {
                return "HECS189";
            }
            if (unit.contains("衡水")) {
                return "HECS720";
            }
            if (unit.contains("雄安")) {
                return "HECS728";
            }
            return "HECS018";
        }
        if ("雄安".equals(areaName)) {
            unit = unit.replace("雄安新区", "");
        }
        List<OrganizationPo> l3 = thirdOrgListMap.getOrDefault(areaNo, new ArrayList<>());
        for (OrganizationPo organizationPo : l3) {
            if (unit.contains(organizationPo.getName())) {
                return organizationPo.getId();
            }
        }
        // 市区
        if ("沧州".equals(areaName)) {
            return "D0130911";
        }
        if ("唐山".equals(areaName)) {
            return "D0130202";
        }
        if ("秦皇岛".equals(areaName)) {
            return "D0130302";
        }
        if ("廊坊".equals(areaName)) {
            return "D0131000";
        }
        if ("张家口".equals(areaName)) {
            return "D0130701";
        }
        if ("邢台".equals(areaName)) {
            return "D0130502";
        }
        if ("邯郸".equals(areaName)) {
            return "D0130402";
        }
        if ("保定".equals(areaName)) {
            return "D0130601";
        }
        if ("石家庄".equals(areaName)) {
            return "D0130186";
        }
        if ("承德".equals(areaName)) {
            return "D0130801";
        }
        if ("衡水".equals(areaName)) {
            return "D0133001";
        }
        if ("雄安".equals(areaName)) {
            return "D0130830";
        }
        // 河北省分公司
        return "HE001";
    }

    /**
     * 获取地图匹配的二级组织机构编码，车效大屏
     * @param areaName 二级组织机构名称
     * @param cityName 三级组织机构名称
     */
    public String getAreaNo2(String areaName, String cityName) {
        if (!StringUtils.hasText(areaName)) {
            return "";
        }
        if ("省机动局".equals(areaName) && StringUtils.hasText(cityName)) {
            if (cityName.contains("沧州")) {
                return "180";
            }
            if (cityName.contains("唐山")) {
                return "181";
            }
            if (cityName.contains("秦皇岛")) {
                return "182";
            }
            if (cityName.contains("廊坊")) {
                return "183";
            }
            if (cityName.contains("张家口")) {
                return "184";
            }
            if (cityName.contains("邢台")) {
                return "185";
            }
            if (cityName.contains("邯郸")) {
                return "186";
            }
            if (cityName.contains("保定")) {
                return "187";
            }
            if (cityName.contains("石家庄")) {
                return "188";
            }
            if (cityName.contains("承德")) {
                return "189";
            }
            if (cityName.contains("衡水")) {
                return "720";
            }
            if (cityName.contains("雄安")) {
                return "782";
            }
        }
        if (areaName.contains("沧州")) {
            return "180";
        }
        if (areaName.contains("唐山")) {
            return "181";
        }
        if (areaName.contains("秦皇岛")) {
            return "182";
        }
        if (areaName.contains("廊坊")) {
            return "183";
        }
        if (areaName.contains("张家口")) {
            return "184";
        }
        if (areaName.contains("邢台")) {
            return "185";
        }
        if (areaName.contains("邯郸")) {
            return "186";
        }
        if (areaName.contains("保定")) {
            return "187";
        }
        if (areaName.contains("石家庄")) {
            return "188";
        }
        if (areaName.contains("承德")) {
            return "189";
        }
        if (areaName.contains("衡水")) {
            return "720";
        }
        if (areaName.contains("雄安")) {
            return "782";
        }
        return "";
    }

    /**
     * 获取地市id
     * @param cities 地市列表
     * @param unit 单位
     */
    public String getCityId(List<AreaPo> cities, String unit) {
        if (!StringUtils.hasText(unit)) {
            return "";
        }
        for (AreaPo city : cities) {
            if (unit.contains(city.getShortName())) {
                return city.getAreaId();
            }
        }
        return "";
    }

    /**
     * 获取地区名称
     * @param areaMap 地区map
     * @param id 地区id
     */
    public String getAreaName(Map<String, AreaPo> areaMap, String id) {
        if (!StringUtils.hasText(id)) {
            return "";
        }
        AreaPo areaPo = areaMap.get(id);
        if (null != areaPo) {
            return areaPo.getAreaName();
        }
        return "";
    }

    /**
     * 获取区县id
     * @param districtListMap 区县列表map
     * @param cityId 地市id
     * @param cityName 地市名称
     * @param unit 单位
     */
    public String getDistrictId(Map<String, List<AreaPo>> districtListMap, String cityId, String cityName, String unit) {
        if (!StringUtils.hasText(cityId) || !StringUtils.hasText(cityName) || !StringUtils.hasText(unit)) {
            return "";
        }
        if ("石家庄".equals(cityName)) {
            if (unit.contains("井陉矿区")) {
                return "130107";
            }
            if (unit.contains("井陉")) {
                return "130121";
            }
        }
        if ("雄安".equals(cityName)) {
            unit = unit.replace("雄安新区", "");
        }
        List<AreaPo> districts = districtListMap.get(cityId);
        if (CollectionUtils.isEmpty(districts)) {
            return "";
        }
        for (AreaPo district : districts) {
            if (unit.contains(district.getShortName())) {
                return district.getAreaId();
            }
        }
        return "";
    }
}
